#pragma once
/* -*-c++-*- osgVisual - Copyright (C) 2009-2010 Torben Dannhauer
 *
 * This library is based on OpenSceneGraph, open source and may be redistributed and/or modified under 
 * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or
 * (at your option) any later version.  The full license is in LICENSE file
 * included with this distribution, and on the openscenegraph.org website.
 *
 * osgVisual requires for some proprietary modules a license from the correspondig manufacturer.
 * You have to aquire licenses for all used proprietary modules.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * OpenSceneGraph Public License for more details.
*/

#include "dataIO_cluster.h"

#include <osg/notify>
#include <osg/ArgumentParser>
#include <osgDB/Registry>
#include <osgDB/ReaderWriter>
#include <iostream>
#include <winsock2.h>
#pragma comment ( lib, "WSock32.lib" )

#include "dataIO_clusterUDP_Transmission.h"

namespace osgVisual
{

/**
 * \brief This is a UDP implementation of dataIO_cluster..
 * 
 * This class uses UDP to fullfil the functions of dataIO_cluster.
 * It sends the clusterdata via UDP Broadcast into the network to any listening client.
 * 
 * 
 * \todo Add getHorizonColor to transport it to slave for consistent fog.
 * @author Torben Dannhauer
 * @date  Feb 2010
 */ 
class dataIO_clusterUDP :	public dataIO_cluster
{
public:
	dataIO_clusterUDP();
	virtual ~dataIO_clusterUDP(void);

	void init( osg::ArgumentParser& arguments_, clustermode clusterMode_, osgVisual::dataIO_transportContainer* sendContainer_, bool compressionEnabled_, bool asAscii_ );
	void shutdown();

	bool sendTO_OBJvaluesToSlaves();
	bool readTO_OBJvaluesFromMaster();
	void reportAsReadyToSwap();
	bool waitForSwap();
	bool waitForAllReadyToSwap();
	bool sendSwapCommand();

private:
	bool compressionEnabled;
	osg::ref_ptr<osgDB::Options> readOptions;
	osg::ref_ptr<osgDB::Options> writeOptions;
	osg::ref_ptr<osgDB::ReaderWriter> rw;
	osg::ref_ptr<osgVisual::Broadcaster> broadcaster;
	osg::ref_ptr<osgVisual::Receiver> receiver;
};

}	// END NAMESPACE